/////////////////////////////////////////////////////////////////////////////// 
//
// Copyright  2012 Ali Mayyasi. All Rights Reserved.
// Written by Ali Mayyasi
//
// DESCRIPTION: This script takes a folder with an ordered sequence of images and 
//				assembles them into a single image based on the specified number of 
//				columns. If the number of columns is left empty, the sequence will be
//				assembled into a perfect square.
//				[https://udn.epicgames.com/Three/FlipbookTextures]
//
/////////////////////////////////////////////////////////////////////////////// 

// enable double clicking  this script file
#target photoshop
app.bringToFront(); //focus Photoshop window
//app.displayDialogs = DialogModes.NO;

/////////////////////////////////////////////////////////////////////////////// 
// Global variables
/////////////////////////////////////////////////////////////////////////////// 
var gExtensionsAllowed = Array( "tga", "psd", "jpg", "bmp");
var gDebug = false;

// ok and cancel buttons
var runButtonID = 1;
var cancelButtonID = 2;

var gDialogInfo; // holds our custom dialog box's info

var strDocumentation = "This script takes a folder with an ordered sequence of images and assembles them into a single image based on the specified number of columns. If the number of columns is left empty, the sequence will be assembled into a perfect square. To abort the script, hit escape. For problems or bugs, email ali.mayyasi@gmail.com"

/////////////////////////////////////////////////////////////////////////////// 
// Aligns a layer based on alignment method
/////////////////////////////////////////////////////////////////////////////// 
function align(method) 
{ 
	// 'AdCH' - center horizontal 
	// 'AdCV' - center vertical 
	// 'AdLf' - left 
	// 'AdRg' - Right 
	// 'AdBt' - bottom 
	// 'AdTp' - Top 
	var desc = new ActionDescriptor();// the command, in this case align 
	var ref = new ActionReference();// What to align, in this case the activeLayer 
	
	ref.putEnumerated( charIDToTypeID( "Lyr " ), charIDToTypeID( "Ordn" ), charIDToTypeID( "Trgt" ) ); 
	desc.putReference( charIDToTypeID( "null" ), ref ); 
	desc.putEnumerated( charIDToTypeID( "Usng" ), charIDToTypeID( "ADSt" ), charIDToTypeID( method ) );// Which align option to use 
	
	executeAction( charIDToTypeID( "Algn" ), desc, DialogModes.NO ); 
}; 

/////////////////////////////////////////////////////////////////////////////// 
// Centers the selected layer 
/////////////////////////////////////////////////////////////////////////////// 
function center(doc) 
{ 
	var docRef = doc;
	docRef.selection.selectAll(); 
	align("AdCH"); 
	align("AdCV"); 
	docRef.selection.deselect(); 
}; 

/////////////////////////////////////////////////////////////////////////////// 
// Is input an integer?
/////////////////////////////////////////////////////////////////////////////// 
function isInt (i)
{
	return (i % 1) == 0;
}

/////////////////////////////////////////////////////////////////////////////// 
// Initializes the startup dialog box
/////////////////////////////////////////////////////////////////////////////// 
function initDialog()
{
	dlgMain = new Window("dialog", "TG Sequence To SubUV Generator");
	dlgMain.orientation = 'column'; 
	dlgMain.alignChildren = 'center';
	
	// the bottom of the dialog
	dlgMain.grpHeader = dlgMain.add("group");
	dlgMain.grpHeader.orientation = 'column';
	dlgMain.grpHeader.alignChildren = 'center';
	dlgMain.grpHeader.alignment = 'fill';

	dlgMain.pnlHelp = dlgMain.grpHeader.add("panel");
	dlgMain.pnlHelp.alignment = 'fill';

	dlgMain.etHelp = dlgMain.pnlHelp.add("statictext", undefined, strDocumentation, {multiline:true});
	dlgMain.etHelp.alignment = 'fill';
	
	// input folder section
	dlgMain.pnlInputPaths = dlgMain.add("panel");
	dlgMain.pnlInputPaths.orientation = 'column';
	dlgMain.pnlInputPaths.alignChildren = 'top';
	dlgMain.pnlInputPaths.alignment = 'fill';
	//dlgMain.pnlInputPaths.text = "Specify Paths:";
	
	dlgMain.grpLine1 = dlgMain.pnlInputPaths.add("group");
	dlgMain.grpLine1.orientation = 'row';
	dlgMain.grpLine1.alignChildren = 'center';
	dlgMain.grpLine1.alignment = 'fill';
	dlgMain.grpLine1.add("statictext", undefined, "Folder with image sequence:");
	
	// browse for input button
	dlgMain.grpLine1b = dlgMain.pnlInputPaths.add("group");
	dlgMain.grpLine1b.orientation = 'row';
	dlgMain.grpLine1b.alignChildren = 'center';
	dlgMain.grpLine1b.alignment = 'fill';
	dlgMain.btnBrowse = dlgMain.grpLine1b.add("button", undefined, "Browse");
	dlgMain.etInputFolder = dlgMain.grpLine1b.add("edittext", undefined, "");
	dlgMain.etInputFolder.preferredSize.width = 300;
	dlgMain.btnBrowse.onClick = btnBrowseInputOnClick;
	
	// file formats dropdown list
	dlgMain.grpLine1c = dlgMain.pnlInputPaths.add("group");
	dlgMain.grpLine1c.orientation = 'row';
	dlgMain.grpLine1c.alignChildren = 'center';
	dlgMain.grpLine1c.alignment = 'fill';
	dlgMain.grpLine1c.add("statictext", undefined, "File format to look for:");
	dlgMain.ddlDropDownlist = dlgMain.grpLine1c.add("dropdownlist", undefined,"test"); 

	for (var i=0; i < gExtensionsAllowed.length; i++)
		dlgMain.ddlDropDownlist.add("item", gExtensionsAllowed[i]);
	dlgMain.ddlDropDownlist.selection = dlgMain.ddlDropDownlist.items[0]
	
	//Tiling input
	dlgMain.grpLine2 = dlgMain.pnlInputPaths.add("group");
	dlgMain.grpLine2.orientation = 'row';
	dlgMain.grpLine2.alignChildren = 'top';
	dlgMain.grpLine2.alignment = 'fill';
	dlgMain.grpLine2.add("statictext", undefined, "Specify the number of columns: ");
	dlgMain.etColumns = dlgMain.grpLine2.add("edittext", undefined,  undefined);
	dlgMain.etColumns.preferredSize.width = 50;
	dlgMain.grpLine2.add("statictext", undefined, "(Optional)");

	// the right side of the dialog, the ok and cancel buttons
	dlgMain.grpLine4 = dlgMain.add("group");
	dlgMain.grpLine4.orientation = 'row';
	dlgMain.grpLine4.alignChildren = 'right';
	dlgMain.grpLine4.alignment = 'fill';
	dlgMain.btnRun = dlgMain.grpLine4.add("button", undefined, "Run");
	dlgMain.btnRun.onClick = btnRunOnClick;
	dlgMain.btnCancel = dlgMain.grpLine4.add("button", undefined, "Cancel" );
	dlgMain.btnCancel.onClick = btnCancelOnClick;
	
	dlgMain.defaultElement = dlgMain.btnRun;
	dlgMain.cancelElement = dlgMain.btnCancel;

	dlgMain.center();
	dlgMain.show();
	
	return;
}

///////////////////////////////////////////////////////////////////////////////
// Object holding the dialog info
///////////////////////////////////////////////////////////////////////////////
function DialogInfo()
{
    this.inputFolder = null;
    this.outputFolder = null;
    this.isSquare = true;
    this.columns = 0;
    this.strFileFormat = "TGA";
}

///////////////////////////////////////////////////////////////////////////////
// Getters and Setters
///////////////////////////////////////////////////////////////////////////////
function setInputFolder(fInputFolder){gDialogInfo.inputFolder = fInputFolder;}
function setOutputFolder(fOutputFolder){gDialogInfo.outputFolder = fOutputFolder;}
function setSquare(bIsSquare){gDialogInfo.isSquare = bIsSquare;}
function setColumns(iNumColumns){gDialogInfo.columns = iNumColumns;}
function setFileFormat(strFormat){gDialogInfo.strFileFormat = strFormat;}

function getInputFolder(){return gDialogInfo.inputFolder;}
function getOutputFolder(){return gDialogInfo.outputFolder;}
function getSquare(){return gDialogInfo.isSquare;}
function getColumns(){return gDialogInfo.columns;}
function getFileFormat(){return gDialogInfo.strFileFormat;}

/////////////////////////////////////////////////////////////////////////////// 
// "Run" button event handler
// Validates input before processing
/////////////////////////////////////////////////////////////////////////////// 
function btnRunOnClick()
{
	if ( !bIsInputValid() )
		return;
		
	// find the dialog and close it
	var d = this;
	while (d.type != 'dialog')
		d = d.parent;
	d.close(runButtonID);
	
	processInput();
}

/////////////////////////////////////////////////////////////////////////////// 
// "Cancel" button event handler
/////////////////////////////////////////////////////////////////////////////// 
function btnCancelOnClick()
{
	// find the dialog
	var d = this;
	while (d.type != 'dialog')
		d = d.parent;
	d.close(cancelButtonID); 
	
	dlgMain.defaultElement = dlgMain.btnRun;
	dlgMain.cancelElement = dlgMain.btnCancel; 
}	

///////////////////////////////////////////////////////////////////////////////
// "Browse for output" button event handler
///////////////////////////////////////////////////////////////////////////////
function btnBrowseOutputOnClick()
{
	var outputFolder = Folder.selectDialog("Select the folder where the output will be saved:", "*.TXT,All files:*");
    if ( outputFolder != null )
	{
        dlgMain.etOutputFolder.text = outputFolder.fsName;
		setOutputFolder(outputFolder);
    }
	dlgMain.defaultElement.active = true;
    return;
}

///////////////////////////////////////////////////////////////////////////////
// "Browse for input" button event handler
///////////////////////////////////////////////////////////////////////////////
function btnBrowseInputOnClick()
{
	var inputFolder = Folder.selectDialog("Select the folder containing your input files:", "*.TXT,All files:*");

    if ( inputFolder != null )
	{
        dlgMain.etInputFolder.text = inputFolder.fsName;
		setInputFolder(inputFolder);
    }
	dlgMain.defaultElement.active = true;
	return;
}

///////////////////////////////////////////////////////////////////////////////
// Checks if any files are open, propmts for a Saveless close, and renders the input dialog box
/////////////////////////////////////////////////////////////////////////////// 
function main()
{
	if ( app.documents.length > 0 )
	{
		if (confirm("All open documents should be closed before running this script. Close them (without saving) and resume?"))
		{
			while (app.documents.length > 0)
			{
				app.activeDocument = app.documents[0];
				app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
			}
		}
		else
			return;
	}
	
	var oldRulerPreferences = app.preferences.rulerUnits; // temporarily store the ruler preference before script clobbers it, restore before script exit
	app.preferences.rulerUnits = Units.PIXELS;
		
	// debug level: 0-2 (0:disable, 1:break on error, 2:break at beginning)
	$.level = 1;
	debugger;

	gDialogInfo = new DialogInfo();
	initDialog();
	app.preferences.rulerUnits = oldRulerPreferences; //restores startup ruler preference
}

/////////////////////////////////////////////////////////////////////////////// 
// validate user input
/////////////////////////////////////////////////////////////////////////////// 
function bIsInputValid()
{
	try
	{
		var numColumns = 0;
		var inputFolder = getInputFolder();
		var outputFolder = getOutputFolder();

		// validate the paths
		if (inputFolder == null)
		{
			alert("Specify the input folder.");
			return false;
		}
		if (outputFolder == null)
			outputFolder="";
		
		// validate the tile options
		if ( !dlgMain.etColumns.text == "" )
		{
			numColumns = dlgMain.etColumns.text;
			numColumns = parseInt(numColumns);
			if ( isNaN(numColumns) || numColumns < 1 )
			{
				alert("Invalid number of column. Positive intiger expected.");
				return false;
			}
			setSquare(false);
		}
		else
		{
			setSquare(true);
			numColumns = -1; //to be set once all files are opened
		}
		
		setColumns(numColumns);
		setFileFormat(dlgMain.ddlDropDownlist.selection.toString())
	}
	catch(e)
	{
		alert("Error validating input: " + e)
		return false;
	}	
	return true;
}

/////////////////////////////////////////////////////////////////////////////// 
// Input to this has already been validated
/////////////////////////////////////////////////////////////////////////////// 
function processInput()
{
	var resolusion = 72;
	var numDocs = 0;
	var doc;

	var inputFolder = getInputFolder();
	var outputFolder = getOutputFolder();
	var numColumns = getColumns();
	var isSquare = getSquare();
	var numRows;
	var fileList;
	
    try
	{
		fileList = inputFolder.getFiles("*."+getFileFormat());
		numDocs = fileList.length;
		
		if ( numDocs == 0 )
		{
			alert("No "+ getFileFormat().toUpperCase() +"s found. Exiting.")
			return;
		}
		
		//Prompt for number of columns, otherwise, assume a perfect square
		if ( isSquare )
		{
			numRows = Math.sqrt(numDocs);
			numColumns = numRows;
			if ( !isInt(numColumns) )
			{
				alert("The number of "+getFileFormat().toUpperCase()+"s in your sequence ("+numDocs+") is not sufficient for a square assembly. It needs to be a power of 2. Exiting.");
				if (gDebug)
				{
					alert("numDocs: " + numDocs);
					alert("numRows:" + numRows);
					alert("numColumns: " + numColumns);
				}
				return;
			}
		}
		else
		{
			numRows = numDocs/numColumns;
			if (gDebug)
			{
				alert("numDocs: " + numDocs);
				alert("numRows:" + numRows);
				alert("numColumns: " + numColumns);
			}
			if ( !isInt(numRows) )
			{
				alert("The number of "+getFileFormat().toUpperCase()+"s in your sequence ("+numDocs+") is not a multiple of " + numColumns +". Exiting.");
				return;
			}
		}
		for (var i = 0; i < fileList.length; i++)
	    {
	        // Make sure all the files in the folder are compatible with PS
	        if ( fileList[i] instanceof File && !fileList[i].hidden )
			{
	            open(fileList[i]);
			}
	    }
		
		//grab dimension of a doc, assuming all of them match
		docWidth  = app.documents[0].width.value
		docHeight = app.documents[0].height.value
		
		// create a new file for RGB
		var docRGB = app.documents.add(docWidth, docHeight, resolusion, null, NewDocumentMode.RGB, DocumentFill.TRANSPARENT);
		docRGB.artLayers.add();//this doc will be the last in the open docs list
		var activeChannels = app.activeDocument.activeChannels

		// create a new file for alphas
		var docAlpha = app.documents.add(docWidth, docHeight, resolusion, null, NewDocumentMode.RGB, DocumentFill.TRANSPARENT);
		docAlpha.artLayers.add();//this doc will be the last in the open docs list
		var activeChannels = app.activeDocument.activeChannels

		/*
		if (app.documents[0].channels.length == 4 )
		{
			alert("Has alpha");
			app.activeDocument = app.documents[0];
			app.activeDocument.selection.selectAll();
			app.activeDocument.selection.copy();
			app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
			doc.paste();
		}
		else
			alert("Skipping alpha");
		*/	
			
		while (app.documents.length > 2)
		{
			//set first most doc as active; copy RGB
			app.activeDocument = app.documents[0];
			app.activeDocument.selection.selectAll();
			app.activeDocument.selection.copy();
		
			//Switch to RGB aggregator doc, paste
			app.activeDocument = app.documents[app.documents.length-2];
			docRGB.paste();
			
			//switch to source doc, and copy alpha
			app.activeDocument = app.documents[0];
			activeChannel = app.documents[0].channels[3];
			app.activeDocument.selection.selectAll();
			app.activeDocument.selection.copy();
			app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);
			
			//Switch to Alpha aggregator doc, paste
			app.activeDocument = app.documents[app.documents.length-1];
			docAlpha.paste();
		}
		//remove first empty layer
		app.activeDocument = app.documents[0];
		docRGB.layers[app.activeDocument.layers.length-1].remove();
		app.activeDocument = app.documents[1];
		docAlpha.layers[app.activeDocument.layers.length-1].remove();

		//resize current doc
		app.activeDocument = app.documents[0];
		docRGB.resizeCanvas(docWidth*numColumns,docHeight*numRows,AnchorPosition.TOPLEFT);
		app.activeDocument = app.documents[1];
		docAlpha.resizeCanvas(docWidth*numColumns,docHeight*numRows,AnchorPosition.TOPLEFT);

		var layerIndex = 0;
		var colIndex = 0;
		var rowIndex = 0;
		var widthOffset = 0;
		var heightOffset = 0;
				
		while (layerIndex < numDocs )
		{
			widthOffset=colIndex*docWidth;
			heightOffset=rowIndex*docHeight;
			app.activeDocument = app.documents[0];
			docRGB.layers[numDocs-layerIndex-1].translate(widthOffset,heightOffset);
			app.activeDocument = app.documents[1];
			docAlpha.layers[numDocs-layerIndex-1].translate(widthOffset,heightOffset);

			if ( colIndex < numColumns-1)
				colIndex++;
			else
			{
				colIndex=0;
				rowIndex++;
			}
			layerIndex++;
		}
		app.activeDocument = app.documents[0];
		docRGB.flatten()
		docRGB.activeLayer.isBackgroundLayer = true;
		app.activeDocument = app.documents[1];
		docAlpha.flatten()
		docAlpha.activeLayer.isBackgroundLayer = true;
		docAlpha.selection.selectAll();
		docAlpha.selection.copy();
		docAlpha.close(SaveOptions.DONOTSAVECHANGES);
		app.activeDocument = app.documents[0];
		
		docRGB.channels.add();
		docRGB.paste();
		
		var RGBChannels = new Array(docRGB.channels[0], docRGB.channels[1], docRGB.channels[2])
		docRGB.activeChannels = RGBChannels
		docRGB.selection.deselect();
		alert("Assembly complete!");
	}
	catch(e)
	{
		alert("Unexpected error: " + e);
	}
	return;
}

/////////////////////////////////////////////////////////////////////////////// 
main();